@@ -23,10 +23,10 @@ import subprocess
 import tarfile
 import zipfile
 
-from copy import deepcopy
 import doctest
 import os
-from unittest import TestSuite
+
+from bzrlib import tests
 
 try:
     from debian.changelog import Version, Changelog
@@ -34,11 +34,23 @@ except ImportError:
     # Prior to 0.1.15 the debian module was called debian_bundle
     from debian_bundle.changelog import Version, Changelog
 
-from bzrlib.tests import TestUtil, multiply_tests, TestCaseWithTransport
+from bzrlib.tests import TestUtil, multiply_tests
+try:
+    from bzrlib.tests.features import (
+        ExecutableFeature,
+        ModuleAvailableFeature,
+        UnicodeFilenameFeature,
+        )
+except ImportError: # bzr < 2.5
+    from bzrlib.tests import (
+        ExecutableFeature,
+        ModuleAvailableFeature,
+        UnicodeFilenameFeature,
+        )
 
 
 def make_new_upstream_dir(source, dest):
-    os.rename(source, dest)
+    shutil.copytree(source, dest)
 
 
 def make_new_upstream_tarball(source, dest):
@@ -47,7 +59,6 @@ def make_new_upstream_tarball(source, dest):
         tar.add(source)
     finally:
         tar.close()
-    shutil.rmtree(source)
 
 
 def make_new_upstream_tarball_bz2(source, dest):
@@ -56,7 +67,6 @@ def make_new_upstream_tarball_bz2(source, dest):
         tar.add(source)
     finally:
         tar.close()
-    shutil.rmtree(source)
 
 
 def make_new_upstream_tarball_zip(source, dest):
@@ -70,7 +80,6 @@ def make_new_upstream_tarball_zip(source, dest):
                 zip.write(os.path.join(dirpath, name))
     finally:
         zip.close()
-    shutil.rmtree(source)
 
 
 def make_new_upstream_tarball_bare(source, dest):
@@ -79,36 +88,30 @@ def make_new_upstream_tarball_bare(source, dest):
         tar.add(source)
     finally:
         tar.close()
-    shutil.rmtree(source)
 
 
-tarball_functions = [('dir', make_new_upstream_dir, '../package-0.2'),
-                     ('.tar.gz', make_new_upstream_tarball,
-                      '../package-0.2.tar.gz'),
-                     ('.tar.bz2', make_new_upstream_tarball_bz2,
-                      '../package-0.2.tar.bz2'),
-                     ('.zip', make_new_upstream_tarball_zip,
-                      '../package-0.2.zip'),
-                     ('.tar', make_new_upstream_tarball_bare,
-                      '../package-0.2.tar'),
-                     ]
+def make_new_upstream_tarball_xz(source, dest):
+    uncompressed = dest + ".temp"
+    tar = tarfile.open(uncompressed, 'w')
+    try:
+        tar.add(source)
+    finally:
+        tar.close()
+    subprocess.check_call(["xz", "-z", uncompressed])
+    os.rename(uncompressed+".xz", dest)
 
 
-class RepackTarballAdaptor(object):
-
-    def adapt(self, test):
-        result = TestSuite()
-        for (name, function, source) in tarball_functions:
-            new_test = deepcopy(test)
-            source = os.path.basename(source)
-            new_test.build_tarball = function(source)
-            new_test.old_tarball = source
-            def make_new_id():
-                new_id = '%s(%s)' % (test.id(), name)
-                return lambda: new_id
-            new_test.id = make_new_id()
-            result.addTest(new_test)
-        return result
+def make_new_upstream_tarball_lzma(source, dest):
+    import lzma
+    f = lzma.LZMAFile(dest, 'w', options={'format': 'alone'})
+    try:
+        tar = tarfile.open(None, 'w', f)
+        try:
+            tar.add(source)
+        finally:
+            tar.close()
+    finally:
+        f.close()
 
 
 def load_tests(standard_tests, module, loader):
@@ -119,12 +122,15 @@ def load_tests(standard_tests, module, loader):
             'test_bzrtools_import',
             'test_commit_message',
             'test_config',
+            'test_dep3',
             'test_dh_make',
             'test_hooks',
             'test_import_dsc',
             'test_merge_changelog',
             'test_merge_package',
+            'test_merge_quilt',
             'test_merge_upstream',
+            'test_quilt',
             'test_repack_tarball_extra',
             'test_revspec',
             'test_source_distiller',
@@ -148,6 +154,12 @@ def load_tests(standard_tests, module, loader):
                               old_tarball='../package-0.2.tar.gz')),
                  ('.tar.bz2', dict(build_tarball=make_new_upstream_tarball_bz2,
                               old_tarball='../package-0.2.tar.bz2')),
+                 ('.tar.xz', dict(build_tarball=make_new_upstream_tarball_xz,
+                              old_tarball='../package-0.2.tar.xz',
+                              _test_needs_features=[XzFeature])),
+                 ('.tar.lzma', dict(build_tarball=make_new_upstream_tarball_lzma,
+                              old_tarball='../package-0.2.tar.lzma',
+                              _test_needs_features=[LzmaFeature])),
                  ('.zip', dict(build_tarball=make_new_upstream_tarball_zip,
                               old_tarball='../package-0.2.zip')),
                  ('.tar', dict(build_tarball=make_new_upstream_tarball_bare,
@@ -157,6 +169,28 @@ def load_tests(standard_tests, module, loader):
     return suite
 
 
+class TestCaseWithTransport(tests.TestCaseWithTransport):
+
+    if not getattr(tests.TestCaseWithTransport, "assertPathDoesNotExist", None):
+        # Compatibility with bzr < 2.4
+        def assertPathDoesNotExist(self, path):
+            self.failIfExists(path)
+
+        def assertPathExists(self, path):
+            self.failUnlessExists(path)
+
+
+class TestCaseInTempDir(tests.TestCaseInTempDir):
+
+    if not getattr(tests.TestCaseInTempDir, "assertPathDoesNotExist", None):
+        # Compatibility with bzr < 2.4
+        def assertPathDoesNotExist(self, path):
+            self.failIfExists(path)
+
+        def assertPathExists(self, path):
+            self.failUnlessExists(path)
+
+
 class BuilddebTestCase(TestCaseWithTransport):
 
     package_name = 'test'
@@ -238,13 +272,27 @@ class SourcePackageBuilder(object):
     >>> builder.dsc_name()
     """
 
-    def __init__(self, name, version, native=False, version3=False):
+    def __init__(self, name, version, native=False, version3=False,
+            multiple_upstream_tarballs=None):
+        """
+        :param name: Package name
+        :param version: Package version
+        :param native: Whether to build a native source package
+        :param version3: Whether to build a version 3.0 source package
+        :param multiple_upstream_tarballs: A list of each top-level directory
+            within the upstream tree which is to be packed as a source format
+            3.0 (quilt) additional upstream tarball
+        """
         self.upstream_files = {}
         self.upstream_symlinks = {}
         self.debian_files = {}
         self.name = name
         self.native = native
         self.version3 = version3
+        self.multiple_upstream_tarballs = multiple_upstream_tarballs
+        if multiple_upstream_tarballs and not (version3 and not native):
+            raise AssertionError("Multiple upstream tarballs are only "
+                    "possible with 3.0 (quilt) format")
         self._cl = Changelog()
         self.new_version(version)
 
@@ -275,6 +323,9 @@ class SourcePackageBuilder(object):
         text = """Source: %s\nSection: misc\n""" % self.name
         text += "Priority: optional\n"
         text += "Maintainer: Maintainer <nobody@ubuntu.com>\n"
+        text += "\n"
+        text += "Package: %s\n" % self.name
+        text += "Architecture: all\n\n"
         self.add_debian_file("debian/control", text)
 
     def new_version(self, version, change_text=None):
@@ -355,6 +406,20 @@ class SourcePackageBuilder(object):
                 cmd = ["dpkg-source", "-sn", "-b", basedir]
         else:
             if not self.native:
+                if self.multiple_upstream_tarballs:
+                    for part in self.multiple_upstream_tarballs:
+                        tar_path = "%s_%s.orig-%s.tar.%s" % (self.name,
+                                self._cl.version.upstream_version,
+                                part, tar_format)
+                        if os.path.exists(tar_path):
+                            os.unlink(tar_path)
+                        tar = tarfile.open(tar_path, 'w:%s' % tar_format)
+                        part_basedir = os.path.join(basedir, part)
+                        try:
+                            tar.add(part_basedir, arcname=part)
+                        finally:
+                            tar.close()
+                        shutil.rmtree(part_basedir)
                 tar_path = "%s_%s.orig.tar.%s" % (self.name,
                         self._cl.version.upstream_version, tar_format)
                 if os.path.exists(tar_path):
@@ -382,3 +447,7 @@ class SourcePackageBuilder(object):
         assert ret == 0, "dpkg-genchanges failed, output:\n%s" % \
                 (proc.stdout.read(),)
         shutil.rmtree(basedir)
+
+
+XzFeature = ExecutableFeature("xz")
+LzmaFeature = ModuleAvailableFeature("lzma")
