@@ -33,7 +33,7 @@ class TestBuilddeb(BuilddebTestCase):
     """Create an unpacked source tree in a branch. Return the working tree"""
     tree = self.make_branch_and_tree('.')
     cl_file = 'debian/changelog'
-    source_files = ['debian/'] + [cl_file]
+    source_files = ['README', 'debian/'] + [cl_file]
     self.build_tree(source_files)
     c = self.make_changelog()
     self.write_changelog(c, cl_file)
@@ -50,14 +50,14 @@ class TestBuilddeb(BuilddebTestCase):
     if isinstance(files, basestring):
       files = [files]
     for filename in files:
-      self.failUnlessExists(os.path.join(build_dir, filename))
+      self.assertPathExists(os.path.join(build_dir, filename))
 
   def assertNotInBuildDir(self, files):
     build_dir = self.build_dir()
     if isinstance(files, basestring):
       files = [files]
     for filename in files:
-      self.failIfExists(os.path.join(build_dir, filename))
+      self.assertPathDoesNotExist(os.path.join(build_dir, filename))
 
   def test_builddeb_registered(self):
     self.run_bzr("builddeb --help")
@@ -66,8 +66,7 @@ class TestBuilddeb(BuilddebTestCase):
     self.run_bzr("bd --help")
 
   def test_builddeb_not_package(self):
-    self.run_bzr_error(['Could not find changelog'], 'builddeb '
-            '--no-user-conf')
+    self.run_bzr_error(['Could not find changelog'], 'builddeb')
 
   def build_really_simple_tree(self):
     tree = self.make_unpacked_source()
@@ -78,33 +77,63 @@ class TestBuilddeb(BuilddebTestCase):
     tree.add([self.uncommited_file])
     return tree
 
+  def build_tree_with_conflict(self):
+    tree = self.make_unpacked_source()
+    self.build_tree([self.commited_file, self.uncommited_file,
+                     self.unadded_file])
+    tree.add([self.commited_file])
+    tree.commit("one", rev_id='revid1')
+    newtree = tree.bzrdir.sprout('newtree').open_workingtree()
+    tree.add([self.uncommited_file])
+    tree.commit("two", rev_id='revid2')
+
+    p = '%s/work/newtree/%s' % (self.test_base_dir, self.uncommited_file)
+    fh = open(p, 'w')
+    fh.write('** This is the conflicting line.')
+    fh.close()
+    newtree.add([self.uncommited_file])
+    newtree.commit("new-two", rev_id='revidn2')
+    conflicts = tree.merge_from_branch(newtree.branch) 
+
+    return (conflicts, tree)
+
   def test_builddeb_uses_working_tree(self):
     self.build_really_simple_tree()
-    self.run_bzr("builddeb --no-user-conf --native --builder true "
-            "--dont-purge")
+    self.run_bzr("builddeb --native --builder true --dont-purge")
     self.assertInBuildDir([self.commited_file, self.uncommited_file])
     self.assertNotInBuildDir([self.unadded_file])
 
+  def test_builddeb_refuses_tree_with_conflicts(self):
+    (conflicts, tree) = self.build_tree_with_conflict()
+    self.assertTrue(conflicts > 0)
+    self.run_bzr_error(
+      ['There are conflicts in the working tree. You must resolve these'],
+      "builddeb --native --builder true --dont-purge")
+
+  def test_builddeb_strict(self):
+    tree = self.build_really_simple_tree()
+    self.run_bzr_error(
+      ['bzr: ERROR: Build refused because there are unknown files in the tree. '
+       "To list all known files, run 'bzr unknowns'."],
+      "builddeb --strict")
+
   def test_builddeb_uses_revision_when_told(self):
     self.build_really_simple_tree()
-    self.run_bzr("builddeb --no-user-conf "
-            "--native --builder true --dont-purge -r-1")
+    self.run_bzr("builddeb --native --builder true --dont-purge -r-1")
     self.assertInBuildDir([self.commited_file])
     self.assertNotInBuildDir([self.unadded_file, self.uncommited_file])
 
   def test_builddeb_error_on_two_revisions(self):
     tree = self.make_unpacked_source()
     self.run_bzr_error(['--revision takes exactly one revision specifier.'],
-                       "builddeb --no-user-conf --native --builder "
-                       "true -r0..1")
+                       "builddeb --native --builder true -r0..1")
 
   def test_builddeb_allows_building_revision_0(self):
     self.build_really_simple_tree()
     # This may break if there is something else that needs files in the
     # branch before the changelog is looked for.
     self.run_bzr_error(['Could not find changelog'],
-                       "builddeb --no-user-conf --native --builder true "
-                       "--dont-purge -r0")
+                       "builddeb --native --builder true --dont-purge -r0")
     self.assertNotInBuildDir([self.commited_file, self.unadded_file,
                               self.uncommited_file])
 
@@ -119,10 +148,14 @@ class TestBuilddeb(BuilddebTestCase):
 
   def test_builder(self):
     tree = self.make_unpacked_source()
-    self.run_bzr('bd --no-user-conf --dont-purge --native --builder '
-            '"touch built"')
+    self.run_bzr('bd --dont-purge --native --builder "touch built"')
     self.assertInBuildDir('built')
 
+  def test_package_merge_first(self):
+    tree = self.make_unpacked_source()
+    self.make_upstream_tarball()
+    self.run_bzr('bd --package-merge --builder true')
+
   def test_hooks(self):
     tree = self.make_unpacked_source()
     self.make_upstream_tarball()
@@ -134,8 +167,38 @@ class TestBuilddeb(BuilddebTestCase):
     finally:
       f.close()
     self.run_bzr('add .bzr-builddeb/default.conf')
-    self.run_bzr('bd --no-user-conf --dont-purge --builder true')
-    self.failUnlessExists('pre-export')
+    self.run_bzr('bd --dont-purge --builder true')
+    self.assertPathExists('pre-export')
     self.assertInBuildDir(['pre-build', 'post-build'])
 
+  def test_utf8_changelog(self):
+    from bzrlib.plugins import builddeb
+    from bzrlib.msgeditor import hooks
+    if "set_commit_message" not in hooks:
+      self.skip("No set_commit_message hook in bzrlib this old")
+    hooks.install_named_hook("set_commit_message",
+      builddeb.debian_changelog_commit, "Test builddeb set commit msg hook")
+    tree = self.make_unpacked_source()
+    f = open("debian/bzr-builddeb.conf", 'wb')
+    try:
+      f.write("[BUILDDEB]\ncommit-message-from-changelog = true")
+    finally:
+      f.close()
+    tree.add("debian/bzr-builddeb.conf")
+    # The changelog is only used for commit message when it already exists and
+    # is then changed, so need to clear it, commit, then set the contents.
+    open("debian/changelog", "w").close()
+    tree.commit("Prepare for release", rev_id="prerel")
+    # Would be nice to use debian.changelog to make one, but it's changed
+    # across versions as to how it wants non-ascii bytes provided.
+    f = open("debian/changelog", "w")
+    try:
+      f.write("  * \xe2\x80\xa6and another thing")
+    finally:
+      f.close()
+    self.run_bzr(['commit'])
+    branch = tree.branch
+    self.assertEqual(u"\u2026and another thing",
+      branch.repository.get_revision(branch.last_revision()).message)
+
 # vim: ts=2 sts=2 sw=2
